﻿using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Infrastructure.Options;
using Infrastructure.Security;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;
using Microsoft.IdentityModel.Tokens;
using NetTaste;

namespace Infrastructure.Extensions;

public static class AuthenticationSetup
{
    /// <summary>
    /// 配置认证服务，包含jwt认证
    /// </summary>
    /// <param name="services"></param>
    /// <param name="configuration"></param>
    /// <param name="builderOptions">自定义认证服务配置</param>
    /// <returns></returns>
    public static IServiceCollection AddDefaultAuthentication(
        this IServiceCollection services,
        IConfiguration configuration,
        Action<AuthenticationBuilder>? builderOptions = null)
    {
        ArgumentNullException.ThrowIfNull(services);
        ArgumentNullException.ThrowIfNull(configuration);
        var audienceOptions = configuration.GetSection(AudienceOptions.Name).Get<AudienceOptions>();
        if (audienceOptions is null || !audienceOptions.IsEnable)
        {
            return services;
        }

        services.TryAddSingleton<JwtSecurityTokenHandler>();
        services.TryAddScoped(typeof(IUserContext<>), typeof(UserContextBase<>));
        services.TryAddSingleton<DefaultTokenHandler>();
        services.TryAddSingleton<IEncryptionService, EncryptionService>();
        services.TryAddSingleton<IPostConfigureOptions<JwtBearerOptions>, JwtBearerOptionsPostConfigureOptions>();
        services.TryAddSingleton(typeof(ITokenBuilder), typeof(TokenBuilderBase<>));

        var key = configuration["AUDIENCE_KEY"] ?? audienceOptions.Secret;
        ArgumentException.ThrowIfNullOrEmpty(key);
        var buffer = Encoding.UTF8.GetBytes(key);
        var securityKey = new SymmetricSecurityKey(buffer);

        var tokenValidationParameters = new TokenValidationParameters()
        {
            ValidateIssuerSigningKey = true,
            IssuerSigningKey = securityKey,
            ValidIssuer = audienceOptions.Issuer,
            ValidateIssuer = true,
            ValidateAudience = true,
            ValidAudience = audienceOptions.Audience,
            ValidateLifetime = true,
            ClockSkew = TimeSpan.FromSeconds(300),
            RequireExpirationTime = true,
            RoleClaimType = ClaimTypes.Role
        };

        var builder = services.AddAuthentication(options =>
        {
            options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
            options.DefaultChallengeScheme = nameof(DefaultAuthenticationHandler);
            options.DefaultForbidScheme = nameof(DefaultAuthenticationHandler);
        });
        builder.AddScheme<AuthenticationSchemeOptions, DefaultAuthenticationHandler>(
            nameof(DefaultAuthenticationHandler),
            options => { });
        builder.AddJwtBearer(options => { options.TokenValidationParameters = tokenValidationParameters; });
        builderOptions?.Invoke(builder);
        return services;
    }
}