﻿using System.Globalization;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Infrastructure.Utils;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;

namespace Infrastructure.Security;

public interface ITokenBuilder
{
    IList<Claim> GetClaimsFromUserContext<TId>(IUserContext<TId> userContext) where TId : IEquatable<TId>;

    void SetUserContext(TokenValidatedContext context);

    JwtTokenInfo GenerateJwtTokenInfo(IReadOnlyCollection<Claim> claims);
}

public abstract class TokenBuilderBase<TId>(
    JwtOptions jwtOptions,
    JwtSecurityTokenHandler jwtSecurityTokenHandler,
    IEncryptionService encryptionService)
    : ITokenBuilder where TId : IEquatable<TId>
{
    public virtual IList<Claim> GetClaimsFromUserContext<TId>(IUserContext<TId> userContext) where TId : IEquatable<TId>
    {
        var claims = new List<Claim>()
        {
            new(JwtRegisteredClaimNames.UniqueName, userContext.Username),
            new(JwtRegisteredClaimNames.NameId, userContext.Id.ToString() ?? string.Empty),
            new(JwtRegisteredClaimNames.Name, userContext.Name),
            new(JwtRegisteredClaimNames.Email, userContext.Email),
            new(JwtRegisteredClaimNames.Iat,
                EpochTime.GetIntDate(DateTime.Now).ToString(CultureInfo.InvariantCulture),
                ClaimValueTypes.Integer64),
            new(JwtRegisteredClaimNames.Exp, jwtOptions.Expiration.ToString())
        };
        claims.AddRange(userContext.RoleIds.Select(rId => new Claim(ClaimTypes.Role, rId)));
        return claims;
    }

    public virtual void SetUserContext(TokenValidatedContext context)
    {
        var userContext =
            context.HttpContext.RequestServices.GetService(typeof(IUserContext<TId>)) as IUserContext<TId> ??
            throw new NullReferenceException(nameof(IUserContext<TId>));
        var principal = context.Principal ?? throw new NullReferenceException(nameof(context.Principal));
        var idClaim = principal.Claims.First(c => c.Type == JwtRegisteredClaimNames.NameId);
        userContext.Id = (TId)Convert.ChangeType(idClaim.Value, typeof(TId));
        userContext.Username =
            principal.Claims.First(c => c.Type == JwtRegisteredClaimNames.UniqueName).Value;
        userContext.Name = principal.Claims.First(c => c.Type == JwtRegisteredClaimNames.Name).Value;
        userContext.Email = principal.Claims.First(c => c.Type == JwtRegisteredClaimNames.Email).Value;
        userContext.RoleIds = principal.Claims.Where(c => c.Type == ClaimTypes.Role).Select(c => c.Value).ToArray();
        userContext.RemoteIpAddress = context.HttpContext.GetRequestIp()!;
    }

    public virtual JwtTokenInfo GenerateJwtTokenInfo(IReadOnlyCollection<Claim> claims)
    {
        var jwtToken = new JwtSecurityToken(
            issuer: jwtOptions.Issuer,
            audience: jwtOptions.Audience,
            claims: claims,
            notBefore: DateTime.Now,
            expires: DateTime.Now.AddSeconds(jwtOptions.Expiration),
            signingCredentials: jwtOptions.SigningCredentials);
        var token = jwtSecurityTokenHandler.WriteToken(jwtToken);
        return new JwtTokenInfo(encryptionService.Encrypt(token), jwtOptions.Expiration,
            JwtBearerDefaults.AuthenticationScheme);
    }
}